VERSION 5.00
Begin VB.Form LineToLineForm 
   Caption         =   "Line To Line"
   ClientHeight    =   2055
   ClientLeft      =   60
   ClientTop       =   450
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   2055
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
   Begin VB.CommandButton StartButton 
      Caption         =   "Start"
      Height          =   375
      Left            =   240
      TabIndex        =   0
      Top             =   1440
      Width           =   4215
   End
   Begin VB.Label secondIterationDisplay 
      Height          =   255
      Left            =   1680
      TabIndex        =   4
      Top             =   600
      Width           =   2775
   End
   Begin VB.Label Label1 
      Caption         =   "Second Iteration:"
      Height          =   255
      Left            =   240
      TabIndex        =   3
      Top             =   600
      Width           =   1215
   End
   Begin VB.Label FirstIterationDisplay 
      Height          =   255
      Left            =   1680
      TabIndex        =   2
      Top             =   240
      Width           =   2775
   End
   Begin VB.Label FirstIterationLabel 
      Caption         =   "First Iteration:"
      Height          =   255
      Left            =   240
      TabIndex        =   1
      Top             =   240
      Width           =   1215
   End
End
Attribute VB_Name = "LineToLineForm"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'---------------------------------------------------------------------------
'
'  This project demonstrates the line to line timer mode on the U3.
'  For this example, FIO4 and FIO5 should be jumpered to FIO6 and FIO7
'  respectively. If things are working properly, the first iteration
'  and second iterations should read 1325 and 4309 counts respectively.
'
'  Tested on firmware 1.30 and hardware version 1.30
'
'  support@labjack.com
'  Jul 15, 2010
'----------------------------------------------------------------------
'

' Force explicit variable declaration to avoid typos
Option Explicit

' These variables are shared between sub-routines
Dim lngError As Long
Dim strError As String * 256
Dim lngHandle As Long
Dim lngIOType As Long
Dim lngChannel As Long
Dim dblValue As Double

' User defined errors in VB begin with 50000
' We will add 50000 to all LabJack errors so that we can tell
' them apart from normal VB errors
Const MIN_USER_ERROR = 50000
Const DRIVER_CLOCK_BASE = LJ_tc48MHZ ' The value to be passed to the driver
Const DRIVER_CLOCK_DIV = 1 ' The divisor to be passed to the driver
                           ' This constant is only provided in case you expand upon this
                           ' example but it is not in use.
Const CLOCK = 48000000 ' Base / divisor in Hz

' When the form is loaded, the device will be opened and the timers configured
Private Sub Form_Load()

    lngIOType = 0
    lngChannel = 0
    dblValue = 0
    lngHandle = 0

    'Use our custom error handling
    On Error GoTo ErrorMessage
    
    'Open the first found LabJack U3.
    lngError = OpenLabJack(LJ_dtU3, LJ_ctUSB, "1", 1, lngHandle)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR

    'Start by using the pin_configuration_reset IOType so that all
    'pin assignments are in the factory default condition.
    lngError = ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Set the timer/counter pin offset to 6, which will put the first
    'timer/counter on FIO6.
    lngError = AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_COUNTER_PIN_OFFSET, 6, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Use the 48 MHz timer clock base
    'Max pulse of 1365 microseconds as given by the formula: max pulse (ms) = (2^16-1) / clock
    'NOTE: If you change this value, please update the constants above
    lngError = AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, DRIVER_CLOCK_BASE, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Use this next section for a divisor (only for users that expand upon this example)
    'Set the divisor to 48 so the actual timer clock is 1 MHz.
    'NOTE: If you use this section, please update the constants above
    'lngError = AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, 48, 0, 0)
    'If lngError <> 0 Then Err.Raise lngError
    
    'Enable 2 timers.  They will use FIO6 and FIO7.
    lngError = AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Configure Timer 0 and 1 as line to line (req firmware 1.30 or later)
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 0, LJ_tmLINETOLINE, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 1, LJ_tmLINETOLINE, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Set timers to read rising edges
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 0, 1, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 1, 1, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Execute the requests.
    lngError = GoOne(lngHandle)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR

    'Get all the results just to check for errors.
    lngError = GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    While (lngError < LJE_MIN_GROUP_ERROR)
        lngError = GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
        If (Not lngError = LJE_NO_MORE_DATA_AVAILABLE) And lngError <> 0 Then Err.Raise lngError
    Wend

ErrorMessage:
    If Err.Number > 50000 Then ' If this is a labjack error
        ErrorToString Err.Number - 50000, strError
        MsgBox "Function returned LabJackUD Error #" & _
            Str$(Err.Number - 50000) & _
            "  " & _
            strError
    ElseIf Err.Number > 0 Then 'VB error
        MsgBox "VB error: Error #" & _
            Str$(Err.Number) & _
            "  " & _
            strError
    End If
    Exit Sub
    
End Sub

' The start button actually demonstrates the timer's functionality
Private Sub StartButton_Click()

    Dim firstIterationCount As Double
    Dim secondIterationCount As Double
    Dim firstIterationTime As Double
    Dim secondIterationTime As Double
    Dim i As Long
    
    ' Set FIO lines low to begin the demonstration
    
    ' Set FIO4 low
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 4, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Set FIO5 low
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 5, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Execute the requests.
    lngError = GoOne(lngHandle)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR

    'Get all the results just to check for errors.
    lngError = GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    While (lngError < LJE_MIN_GROUP_ERROR)
        lngError = GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
        If (Not lngError = LJE_NO_MORE_DATA_AVAILABLE) And lngError <> 0 Then Err.Raise lngError
    Wend
    
    ' This first example deomonstrates a quick pulse by first setting FIO4 high
    ' and then immediately setting FIO5 high
    
    ' Set FIO4 high to start timer (assumes that FIO4 is jumpered to FIO6)
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 4, 1, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Set FIO5 high to stop timer (assumes that FIO5 is jumpered to FIO7)
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 5, 1, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Execute the requests.
    lngError = GoOne(lngHandle)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR

    'Get all the results just to check for errors
    lngError = GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    While (lngError < LJE_MIN_GROUP_ERROR)
        lngError = GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
        If (Not lngError = LJE_NO_MORE_DATA_AVAILABLE) And lngError <> 0 Then Err.Raise lngError
    Wend
    
    ' Get the reading from Timer0
    lngError = eGet(lngHandle, LJ_ioGET_TIMER, 0, firstIterationCount, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Reset the timer and FIO lines. Please note that, when using line to line
    ' mode, the user must reset the timer before the LabJack will measure again.
    
    ' Set FIO4 low
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 4, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Set FIO5 low
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 5, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Reset the timers
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 0, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 1, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Execute the requests.
    lngError = GoOne(lngHandle)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR

    'Get all the results just to check for errors.
    lngError = GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    While (lngError < LJE_MIN_GROUP_ERROR)
        lngError = GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
        If (Not lngError = LJE_NO_MORE_DATA_AVAILABLE) And lngError <> 0 Then Err.Raise lngError
    Wend
    
    ' This next example demonstrates a longer pulse by setting FIO4 high
    ' repeatedly and then setting FIO5 high.
    
    ' Set FIO4 high
    For i = 0 To 4
        lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 4, 1, 0, 0)
        If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    Next i
    
    ' Set FIO5 high to stop the timer
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 5, 1, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Read from Timer0
    lngError = AddRequest(lngHandle, LJ_ioGET_TIMER, 0, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Execute the requests.
    lngError = GoOne(lngHandle)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR

    'Get all the results
    lngError = GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    While (lngError < LJE_MIN_GROUP_ERROR)
        lngError = GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
        
        ' If it is the result from getting the timer value, save the value
        If lngIOType = LJ_ioGET_TIMER Then secondIterationCount = dblValue
        
        ' Report errors
        If (Not lngError = LJE_NO_MORE_DATA_AVAILABLE) And lngError <> 0 Then Err.Raise lngError
    Wend
    
    ' Calculate the time of the pulse for each iteration
    ' NOTE: if you change the clock, this formula's cosntants will need to be
    '       updated above
    firstIterationTime = firstIterationCount / CLOCK * 1000000
    secondIterationTime = secondIterationCount / CLOCK * 1000000
    
    ' Display results
    FirstIterationDisplay.Caption = Format$(firstIterationTime, "#.### microseconds")
    secondIterationDisplay.Caption = Format$(secondIterationTime, "#.### microseconds")
    
    ' Reset the timer and FIO lines in case you run it again
    
    ' Set FIO4 low
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 4, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Set FIO5 low
    lngError = AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 5, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    ' Reset the timers
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 0, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    lngError = AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 1, 0, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    
    'Execute the requests.
    lngError = GoOne(lngHandle)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR

    'Get all the results just to check for errors.
    lngError = GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
    If lngError <> 0 Then Err.Raise lngError + MIN_USER_ERROR
    While (lngError < LJE_MIN_GROUP_ERROR)
        lngError = GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, 0, 0)
        If (Not lngError = LJE_NO_MORE_DATA_AVAILABLE) And lngError <> 0 Then Err.Raise lngError
    Wend

ErrorMessage:
    If Err.Number > 50000 Then ' If this is a labjack error
        ErrorToString Err.Number - 50000, strError
        MsgBox "Function returned LabJackUD Error #" & _
            Str$(Err.Number - 50000) & _
            "  " & _
            strError
    ElseIf Err.Number > 0 Then 'VB error
        MsgBox "VB error: Error #" & _
            Str$(Err.Number) & _
            "  " & _
            strError
    End If
    Exit Sub
        
End Sub
